package org.jpos.util;

import org.jpos.iso.*;

/**
 * $Revision: 23416 $
 * $Date: 2007-03-14 00:14:09 +0000 (Wed, 14 Mar 2007) $
 * $Author: nsmith $
 */
public class ISOComponentDeepCopier {

    protected static ISOComponentDeepCopier instance = null;
    
    protected ISOBinaryFieldDeepCopier isoBinaryFieldDeepCopier;
    protected ISOBitMapDeepCopier isoBitMapDeepCopier; 
    protected ISOFieldDeepCopier isoFieldDeepCopier; 
    protected ISOMsgDeepCopier isoMsgDeepCopier; 
    
    ISOComponentDeepCopier() {
    }
    
    public ISOComponent doDeepCopy(ISOComponent component) {
        if (component instanceof ISOBinaryField) {
            return isoBinaryFieldDeepCopier.doDeepCopy((ISOBinaryField) component);
        } else if (component instanceof ISOBitMap) {
            return isoBitMapDeepCopier.doDeepCopy((ISOBitMap) component);
        } else if (component instanceof ISOField) {
            return isoFieldDeepCopier.doDeepCopy((ISOField) component);
        } else if (component instanceof ISOMsg) {
            return isoMsgDeepCopier.doDeepCopy((ISOMsg) component);
        } else {
            throw new RuntimeException("There is a kind of ISOComponent that this class does not know how to deep copy");
        }
    }
    
    public static ISOComponentDeepCopier getInstance() {
        if (instance == null) {
            instance = new ISOComponentDeepCopier();
            instance.setIsoBinaryFieldDeepCopier(new ISOBinaryFieldDeepCopier());
            instance.setIsoBitMapDeepCopier(new ISOBitMapDeepCopier());
            instance.setIsoFieldDeepCopier(new ISOFieldDeepCopier());
            instance.setIsoMsgDeepCopier(new ISOMsgDeepCopier(instance));
        }
        return instance;
    }

    void setIsoBinaryFieldDeepCopier(ISOBinaryFieldDeepCopier isoBinaryFieldDeepCopier) {
        this.isoBinaryFieldDeepCopier = isoBinaryFieldDeepCopier;
    }

    void setIsoBitMapDeepCopier(ISOBitMapDeepCopier isoBitMapDeepCopier) {
        this.isoBitMapDeepCopier = isoBitMapDeepCopier;
    }

    void setIsoFieldDeepCopier(ISOFieldDeepCopier isoFieldDeepCopier) {
        this.isoFieldDeepCopier = isoFieldDeepCopier;
    }

    void setIsoMsgDeepCopier(ISOMsgDeepCopier isoMsgDeepCopier) {
        this.isoMsgDeepCopier = isoMsgDeepCopier;
    }
}